$(document).ready(function() {
	// Logging. Great for debugging, Very verbose
	$.fn.wizard.logging = false;

	// Globals
	var session_hosts = [];
	var create_more = false;
	var hosts = [];

	var incHeight = ($(window).height() - 360 - 65) * 0.68;
	if (incHeight < 0) {
		incHeight = 0;
	}

	// Wizard options
	var wizard_options = {
		progressBarCurrent: true,
		width: "80%",
		increaseHeight: incHeight,
		buttons: {
			nextText: _("Next"),
			submitText: _("Save Configuration and View Added Hosts")
		}
	};
	// Initialize wizard
	var wizard = $("#host-wizard").wizard(wizard_options);

/****************************************************************************
 *
 *	COMMON FOR ALL CARDS
 *
 ***************************************************************************/

	// Hide popovers when they are clicked
	wizard.el.find('.wizard-card').on('click', '.popover', function() {
		$(this).hide();
	});

	// Hide the progress bar
	wizard.el.find('.progress-striped').hide();

	// Redirect to TAC if close button is clicked
	wizard.el.find('button.wizard-close').on('click', function() {
		window.location.href = 'tac';
	});

	// What to do when wizard resets
	wizard.on("reset", function(wizard) {
		// Reset elements
		wizard.el.find('input').val('');
		wizard.el.find('button.active').removeClass('active btn-primary').text(_('Select'));
		wizard.el.find('#save-add-more').remove();
		wizard.el.find('#host-detail .wizard-input-section').not(':first').remove();
		$('#configuration-saved-row').hide();
		var alert = $('#configuration-saved-alert');
		alert.removeClass().addClass('alert alert-info');
		alert.text(_(' Configuration Saved'));

		wizard.el.find("#management-packs button.pack-btn").each(function() {
			$(this).data("selected", 0);
		});

		// Reset glabals
		hostData = null;
		hosts = [];
		saved_hosts = [];
		create_more = false;
	});

/****************************************************************************
 *
 *	INTRODUCTION CARD
 *
 ***************************************************************************/

	$('#skip-intro').on('click', function() {
		var checkbox = $(this);
		checkbox.attr('disabled', true);
		checkbox.siblings('.alert').remove();
		var progress = $('<img class="saving_in_progress" src="'+_site_domain+'application/media/images/loading_small.gif" alt="Saving" />');
		progress.insertAfter(checkbox);
		$.ajax({
			url: _site_domain + _index_page + '/wizard/save_ninja_setting/',
			type: 'post',
			data: {
				'setting': checkbox.attr('name'),
				'value': checkbox.prop('checked') ? 1 : 0,
				'csrf_token': _csrf_token
			},
			success: function() {
				progress
					.replaceWith("<span class='alert alert-success'>Setting saved</span>");
			},
			error: function() {
				checkbox
					.prop('checked', !checkbox.prop('checked'));
				progress
					.replaceWith("<span class='alert alert-error'>Failed to save setting</span>");
			},
			complete: function() {
				checkbox.attr('disabled', false);
			}

		});
	});

/****************************************************************************
 *
 *	MANAGEMENT PACK CARD
 *
 ***************************************************************************/

	$('#management-packs').append(
		$('<p />')
			.attr('id', 'loading-packs')
			.text(_('Loading management packs ...'))
	);

	// Fetch management packs
	$.ajax({
		url: '/api/config/management_pack',
		type: 'get',
		data: { 'format': 'json', 'auth_method': 'ninja' },
		success: function(packs) {
			$('#loading-packs').remove()
			var authErrorBox = wizard.el.find('.auth-error');
			authErrorBox.hide();
			var broken_packs = [];
			if (packs.length === 0) {
				authErrorBox.children('.auth-error-heading').text(_('No Management Packs configured'));
				authErrorBox.children('.auth-error-text').text(_('The Wizard is dependent on that at least one Management Pack has been configured.'));
				authErrorBox.show();
				wizard.hideButtons();
				$('.next-paragraph').hide();
				return;
			}
			var man_packs = $("#management-packs");
			var packs_added = 0;
			for (var i in packs) {
				var pack = $('<div class="manpack"/>')
					.data('manpack', packs[i].name)
					.data('text', _('Loading information about ') + packs[i].name)
					.addClass('mgmt-pack-option row-fluid')
					.append($('<h4/>').text(packs[i].name + _(' (loading information)')));
				man_packs
					.append($('<hr/>'))
					.append(pack);
				getManagementPackProperties(packs[i], function(pack, name, xhr) {
					packs_added++;
					if (pack == 'error') {
						console.log(xhr.responseText);
						broken_packs.push(name);
					} else {
						// Get property names for management pack
						var mpProps = objectKeys(pack);
						// Create dom elements for individual management packs
						var mgmt_pack_option = man_packs.find('.manpack').filter(function() {
							return $(this).data('manpack') === name;
						});
						var heading = $('<h4/>').text(pack.management_pack_name);
						var icondiv = $('<div/>').addClass('span2 mw82');
						var descriptionDiv = $('<div/>').addClass('span9').text(pack.description);
						var icon = $('<img/>')
							.attr('src', _site_domain+'application/media/images/logos/'+pack.icon)
							.attr('alt', '');
						var button = '<button type="button" class="btn pack-btn" data-toggle="button" data-pack="'+pack.management_pack_name+'" data-hostgroup="'+pack.hostgroup+'">Select</button>';
						var customVarDiv = $('<div />').attr('id', pack.management_pack_name+'_custom_vars').addClass('control-group row-fluid span11');
						var customVarTable = $('<table/>').addClass('span11 cvTable');

						// Sort management pack properties
						mpProps.sort(function(a,b) {
							if (a < b)
								return -1;
							return a > b;
						});

						// Create dom elements for management packs custom variables
						for (var idx in mpProps) {
							// Is the property a Custom Variable?
							if (mpProps[idx].substring(0,1) === "_") {
								// Escape malicious things
								mpProps[idx] = escape(mpProps[idx]);
								// Strip any _op5*__ prefixes from the custom variable name
								var custom_var_name = mpProps[idx].replace(/^_OP5[A-Z]*[_]{2}/, '');
								// Replace underscore with space
								custom_var_name = custom_var_name.replace(new RegExp('[_]+', 'g'), ' ').toLowerCase();
								// Trim all whitespace from beginning and end of label
								custom_var_name = objectTrim(custom_var_name);

								var row = $('<tr/>');

								// Text input
								var label = '<td class=""><label for="'+mpProps[idx]+'">'+custom_var_name+': </label></td>';
								var input = '<td class=""><input type="text" name="'+mpProps[idx]+'" id="'+mpProps[idx]+'" data-pack="'+pack.management_pack_name+'" data-validate="validateNotEmptyIfChosen" ></td>';
								// Popover help.
								var help = $('<td>').addClass('span1 mw20').append(
									$('<a/>', {
										html: '<img src="'+_site_domain+'/application/views/icons/x16/help.png" height="16px" width="16px">',
										href: "#",
										title: custom_var_name,
										// Make help icons not "tabbable"
										tabindex: "-1"
									}).popover({
										placement: 'right',
										trigger: 'click',
										content: pack[mpProps[idx]]
									})
								);
								customVarTable.append(row.append(label).append(input).append(help));
							}
						}

						mgmt_pack_option
							.empty()
							.append(heading)
							.append(icondiv.append(icon).append('<br>'+button))
							.append(descriptionDiv)
							.append(customVarDiv.append(customVarTable));
					}

					// Have all menegement packs had their dom elements created
					if (packs_added === packs.length) {
						if (broken_packs.length > 0) {
							var authErrorBox = wizard.el.find('.mgmt-pack-error');
							authErrorBox.children('.mgmt-pack-error-heading').text(_('There appears to be broken management pack(s) in your configuration'));
							authErrorBox.children('.mgmt-pack-error-text').text(_('The management pack(s) ') + broken_packs.join(', ') + _(' appears to be broken and needs attention. Contact your administrator.'));
							authErrorBox.show();
						}
					}
				});
			}
		},
		error: function(xhr, status, error) {
			var errorDescription = $.parseJSON(xhr.responseText);
			var authErrorBox = wizard.el.find('.auth-error');
			authErrorBox.children('.auth-error-heading').text(xhr.status + ': ' + xhr.statusText);
			authErrorBox.children('.auth-error-text').text(errorDescription.error + ': ' + fullErrorToText(errorDescription.full_error));
			authErrorBox.show();
			wizard.hideButtons();
		}
	});

	// Select management pack buttons
	wizard.el.find("#management-packs").on('click', 'button.pack-btn', function() {
		var button = $(this);
		if(button.hasClass('active')) {
			// Deselect pack
			button.text(_('Select'));
			button.removeClass('btn-primary');
			button.data('selected', 0);
		} else {
			// Select pack
			button.text(_('Selected'));
			button.addClass('btn-primary');
			button.data('selected', 1);
		}
	});

	// Fix for among others Firefox that remember how far user had scrolled when switching cards, Scroll to top.
	wizard.cards["host-management-packs"].on('selected', function() {
		$('.wizard-card').scrollTop(0);
	});

	// Validate the management pack step. At least one pack must be selected
	wizard.cards["host-management-packs"].on("validate", function(card) {
		var chosen = card.el.find("button.active");
		var alertBox = card.el.find("#mgmt-pack-errors");
		if (chosen.length > 0) {
			alertBox.hide();
			return true;
		}
		alertBox.show();
		alertBox.text(_("You must select at least one Management Pack"));
		// Scroll to top to make the error message visible
		$('.wizard-card').scrollTop(0);
		return false;
	});

/****************************************************************************
 *
 *	HOST NAME/ADDRESS CARD
 *
 ***************************************************************************/

	// Set help on host name and host address inputs
	var host_name_help = "The host name is a label that you put on your host. You can normally use the same name as the host name. The name that you provide is how op5\u00a0Monitor identifies your host and must therefore be unique.";
	var host_address_help = "The host address can either be an IP address or a hostname that your DNS (Domain Name Server) can resolve.";
	wizard.el.find('.host-address-help').popover({
		placement: 'right',
		trigger: 'click',
		content: host_address_help
	});
	wizard.el.find('.host-name-help').popover({
		placement: 'right',
		trigger: 'click',
		content: host_name_help
	});

	// Add Host Row
	wizard.el.find("#add-host-form").on('click', function(event) {
		event.preventDefault();
		var row = $(this).parent().siblings('.wizard-input-section').first().clone();
		row.find('p').remove();
		row.find('input').val('');
		row.find('button.remove-host-form').show();
		$(this).parent().before(row);
		// Add help to created row
		wizard.el.find('.host-address-help').popover({
			placement: 'right',
			trigger: 'click',
			content: host_address_help
		});
		wizard.el.find('.host-name-help').popover({
			placement: 'right',
			trigger: 'click',
			content: host_name_help
		});
		var card = $(this).closest('.wizard-card');
		// Scroll to bottom
		card.scrollTop(10000);
	});

	// Remove Host Row
	wizard.el.find('#host-detail').on('click', 'button.remove-host-form', function(event) {
		$(this).closest('.wizard-input-section').remove();
		// Trigger revalidation of the cards to ensure correct alignment of popovers
		wizard.cards["host-detail"].validate();
	});

	// Focus first input on host name/address card
	wizard.cards["host-detail"].on('selected', function() {
		$('.new-host-name').first().focus();
	});

/****************************************************************************
 *
 *	CONFIGURATION REVIEW CARD
 *
 ***************************************************************************/

	// All host data has been entered. Review and prepare data for submission
	wizard.cards["configuration-review"].on('selected', function() {
		var saveViewHosts = $('.btn-success');
		var btnContainer = $('.wizard-buttons-container');
		saveViewHosts.attr('id', 'save-view-hosts');
		var saveAddMore = $('<button/>').addClass('btn btn-info').attr('id', 'save-add-more').data('loading-text', 'Saving Configuration').text('Save Configuration and Add More Hosts');
		btnContainer.append(saveAddMore);
		var successBox = wizard.el.find('#configuration-review .alert-success');
		var errorBox = wizard.el.find('#configuration-review .alert-error');
		var naReview = wizard.el.find('#configuration-review .hosts-name-address');
		var notifyReview = wizard.el.find('.notification-review');
		// Hide alerts
		successBox.hide();
		errorBox.hide();

		// Get Values from all inputs as a serialized Object
		// hostData[0] = host addresses, host names, hostgroups and custom_vars
		// hostData[1] = Management Packs
		var hostData = getInputs(wizard);

		var hostReview = '';
		var loading, hostAlert;
		if ($.isArray(hostData[0].host_name)) {
			// User has created more than one host.
			for (var i=0;i<hostData[0].host_name.length;i++) {
				// Clone object for each host
				hosts.push($.extend(true, {}, hostData[0]));
				// Set host name and address for object
				hosts[i].host_name = hostData[0].host_name[i];
				hosts[i].address = hostData[0].address[i];
				loading = $('<img/>')
					.attr('src', '/monitor/application/media/images/loading_small.gif')
					.addClass('submit-load')
					.css('display', 'none');
				hostAlert = $('<div/>')
					.addClass('alert alert-info host-status')
					.attr('data-host_name', hostData[0].host_name);

				hostAlert.append(loading).append(' <span>'+hostData[0].host_name[i]+'('+hostData[0].address[i]+')</span>');
				naReview.append(hostAlert);
			}
		} else {
			// Only one host to be created. Just push a reference
			hosts.push(hostData[0]);
			loading = $('<img/>')
					.attr('src', '/monitor/application/media/images/loading_small.gif')
					.addClass('submit-load')
					.css('display', 'none');
			hostAlert = $('<div/>')
				.addClass('alert alert-info host-status')
				.attr('data-host_name', hostData[0].host_name);

			hostAlert.append(loading).append(' <span>'+hostData[0].host_name+'('+hostData[0].address+')</span>');
			naReview.append(hostAlert);
		}
	});

	// User chose another card
	wizard.cards["configuration-review"].on('deselect', function() {
		hosts = [];
		$('#save-add-more').remove();
		$('#configuration-review .hosts-name-address').children().remove();
	});

	// Save Configuration and View Created Hosts clicked
	wizard.el.find('.wizard-buttons-container').on('click', '#save-view-hosts', function(event) {
		event.preventDefault();
		create_more = false;
		wizard.trigger('submit');
	});

	// Save Configuration and Add More Hosts clicked
	wizard.el.find('.wizard-buttons-container').on('click', '#save-add-more', function(event) {
		event.preventDefault();
		create_more = true;
		wizard.trigger('submit');
	});

	wizard.on('submit', function() {
		var saved_hosts = [];
		// Submit hosts to nacoma
		for (var i in hosts) {
			// Submit each host with a callback
			addHost(hosts[i], function(data, xhr, host_name) {
				var hostAlert = $(".host-status[data-host_name='"+host_name+"']");
				if (typeof(xhr.status) !== 'undefined' && xhr.status === 201) {
					hostAlert.children('img.submit-load').hide();
					hostAlert.removeClass('alert-info');
					hostAlert.addClass('alert-success');
					hostAlert.prepend('<span class="label label-success">Success</span>');
					// Push host to saved array for current hosts
					saved_hosts.push(data.host_name);
					// Push host to saved array for session hosts. This is used when user chooses to add more hosts after saving
					session_hosts.push(data.host_name);
					if (saved_hosts.length === hosts.length) {
						$('#configuration-saved-row, img.submit-load').show();
						pushChanges(function(data, status, xhr) {
							var alert = $('#configuration-saved-alert');
							if (typeof(xhr.status) !== 'undefined' && xhr.status === 200) {
								alert.children('img.submit-load').hide();
								alert.removeClass('alert-info');
								alert.addClass('alert-success');
								alert.prepend('<span class="label label-success">Success</span>');
									if (create_more === true) {
										// User wants to add more hosts, reset and go to man pack card
										setTimeout(function() {
											wizard.reset();
											wizard.setCard(1);
										}, 1500);
									} else {
										// User wants to view created hosts, create url from the hosts saved this session
										var base = 'listview?q=';
										var url = '[services] host.name = \"';
										url += session_hosts.join('\" or host.name = \"') + '\"';
										// Redirect
										setTimeout(function() {
											window.location.href = base + encodeURIComponent(url);
										}, 1500);
									}
							} else {
								alert.children('img.submit-load').hide();
								alert.removeClass('alert-info');
								alert.addClass('alert-error');
								alert.prepend('<span class="label label-important">Error</span>');
								var errorDescription = $.parseJSON(xhr.responseText);
								alert.append($('<p>').text(errorDescription.error + ': ' + fullErrorToText(errorDescription.full_error)));
							}
						});
					}
				} else {
					hostAlert.children('img.submit-load').hide();
					hostAlert.removeClass('alert-info');
					hostAlert.addClass('alert-error');
					hostAlert.prepend('<span class="label label-important">Error</span>');
					var errorDescription = $.parseJSON(xhr.responseText);
					hostAlert.append($('<p>').text(errorDescription.error + ': ' + fullErrorToText(errorDescription.full_error)));
				}
			});
		}
	});

	// Start wizard
	wizard.show();

	// User wants to skip intro, check the setting and begin at second card
	if(skip_intro === 1) {
		wizard.setCard(1);
	}
});

/****************************************************************************
 *
 *	Data Retrieval
 *
 ***************************************************************************/

// Save configuration
function pushChanges(callback) {
	$.ajax({
		url: '/api/config/change?auth_method=ninja',
		type: 'post',
		headers: {
			'X-op5-csrf-token': _csrf_token
		},
		dataType: 'json',
		success: function(data, status, xhr) {
			return callback(data, status, xhr);
		},
		error: function(xhr, status, error) {
			return callback(error, status, xhr);
		}
	});
}

// Submit host
function addHost(host, callback) {
	$.ajax({
		// On post the auth_method is not appended to the url automagically
		url: '/api/config/host?auth_method=ninja',
		type: 'post',
		dataType: 'json',
		data: host,
		headers: {
			'X-op5-csrf-token': _csrf_token
		},
		success: function(data, status, xhr) {
			return callback(data, xhr, host.host_name);
		},
		error: function(xhr) {
			return callback('throwaway', xhr, host.host_name);
		}
	});
}


// Get properties for management packs
function getManagementPackProperties(pack, callback) {
	$.ajax({
		url: '/api/config/management_pack/' + pack.name,
		type: 'get',
		data: { 'format': 'json', 'auth_method': 'ninja' },
		success: function(data, status, xhr) {
			return callback(data, pack.name, xhr);
		},
		error: function(xhr, status, error) {
			return callback(status, pack.name, xhr)
		}
	});
}

/****************************************************************************
 *
 *	DATA PREPARATION
 *
 ***************************************************************************/

// Serializes object into json data
$.fn.serializeObject = function() {
	var o = {};
	var a = this.serializeArray();
	$.each(a, function() {
		if (this.name === 'hostgroups') {
			var hg = this.value.split(',');
			o[this.name] = hg;
		} else if (o[this.name]) {
			if (!o[this.name].push) {
				o[this.name] = [o[this.name]];
			}
			o[this.name].push(this.value || '');
		} else {
			o[this.name] = this.value || '';
		}
	});
	return o;
};

// Get values from all inputs and selected management packs
function getInputs(wizard) {
	var buttons = wizard.el.find('button.pack-btn');
	var selectedPacks = [];
	var hostgroups = [];
	// Check which buttons where pressed
	buttons.each(function() {
		var button = $(this);
		var selected = button.data('selected');
		if (selected === 1) {
			selectedPacks.push(button.data('pack'));
			hostgroups.push(button.data('hostgroup'));
		}
	});
	// Filter out the unwanted inputs and serialize the inputs
	var rawData = wizard.el.find('input').filter(function() {
		var input = $(this);
		if (typeof(input.data('pack')) === 'undefined' || $.inArray(input.data('pack'), selectedPacks) > -1) {
			if (input.attr('id') === 'hostgroups') {
				input.val(hostgroups);
				return true;
			} else if (input.attr('id') !== 'skip-intro') {
				return true;
			}
		}
		return false;
	});
	return [rawData.serializeObject(), selectedPacks];
}

/****************************************************************************
 *
 *	VALIDATION
 *
 ***************************************************************************/

// Field must not be empty
function notEmpty(el) {
	var ret = {
		status: true
	};
	if (el.val().length < 1) {
		ret.status = false;
		ret.msg = "You must enter a value";
	}
	return ret;
}

// Validates that hostname is not taken, is empty or contains invalid characters
function validateHostname(el) {
	var not_allowed_chars = "~!$%^&*|'\"<>?,()=";
	var not_allowed = new RegExp("["+not_allowed_chars+"]+", 'g');
	var val = el.val();
	var ret = {
		status: true
	};
	if (val === "") {
		ret.status = false;
		ret.msg = "You must enter a name for this host";
	} else if (not_allowed.test(val)){
		ret.status = false;
		ret.msg = "You cannot have any of these characters in the host name: "+not_allowed_chars;
	} else {
		$.ajax({
			url: '/api/config/host',
			type: 'get',
			async: false,
			data: { 'format': 'json', 'auth_method': 'ninja' },
			success: function(data) {
				for (var i in data) {
					if (data[i].name === val) {
						ret.status = false;
						ret.msg = "A host with that name already exists";
					}
				}
			},
			error: function(xhr, status, error) {
				var errorDescription = $.parseJSON(xhr.responseText);
				var authErrorBox = $('.auth-error');
				authErrorBox.children('.auth-error-heading').text(xhr.status + ': ' + xhr.statusText);
				authErrorBox.children('.auth-error-text').text(errorDescription.error + ': ' + fullErrorToText(errorDescription.full_error));
				authErrorBox.show();
			}
		});
	}
	return ret;
}

// Validates that custom vars fields are not empty if corresponding pack is chosen
function validateNotEmptyIfChosen(el) {
	var val = objectTrim(el.val());
	var validate_wmi_user = new RegExp(/\\/, 'g');
	var ret = {
		status: true
	};
	if (el.closest('div.mgmt-pack-option').find('button.pack-btn').hasClass('active')) {
		// Only active management packs should be checked
		if (val === "") {
			ret.status = false;
			ret.msg = "You must enter a value";
		} else if(el.attr('id') === "_USER" && validate_wmi_user.test(val)) {
			ret.status = false;
			ret.msg = "Supported format user/domain";
		}
	}
	return ret;
}

function fullErrorToText(error) {
	if(typeof error === "object") {
		if(typeof error.type === "string") {
			return error.type;
		}
		// This is a special case, where error.*field*.*type_of_error* = *error_msg*
		// which is very hard to generalize, yet harder to print nicely. Just dumping it,
		// trying not to lie. We've already entered a bad state...
		return JSON.stringify(error);
	}
	return error;
}

/****************************************************************************
 *
 *	INTERNET EXPLODER HACKS
 *
 ***************************************************************************/

function objectKeys(object) {
	var properties = [];
	for (var prop in object) {
		if (object.hasOwnProperty(prop))
			properties.push(prop);
	}
	return properties;
}

function objectTrim(object) {

	return object.replace( /^\s+|\s+$/g , "" );

}
